
#define __STRICT_ANSI__ 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <ezLicenseInfo.h>


int CALLING_CONVENTION checkKeySecure(
							const char *keybuf,
							const char	*userhost,
							char		keycookie[],
							char		newkeycookie[],
							size_t		keycookiebufsize);

/*
*	This function takes the Warnings Bit map as argument and interprets it and 
*	displays corresponding warnings messages.
*	@param iWarningsBitMap : BitMap denoting Warnings/Errors.
*	@return statuscode
*/
int CALLING_CONVENTION checkWarnings(int iWarningsBitMap);
/*
*	This function takes the Warnings Bit map as argument and interprets it and 
*	displays corresponding error messages.
*	@param iWarningsBitMap : BitMap denoting Warnings/Errors.
*	@return statuscode
*/
int CALLING_CONVENTION checkErrors(int iWarningsBitMap);
/*
* This function gets and displays the basic license parameters.
*/
void CALLING_CONVENTION displayLicenseParams(void* ezLicInstance);

char * CALLING_CONVENTION getFileNameSuffix(const char *fname);
int CALLING_CONVENTION getKey(const char *filelines[], int numlines,
							  const char fullbuf[], size_t fullbuflen, 
							  const char *suffix,
							  char keybuf[], size_t keybufsize);
int CALLING_CONVENTION readFile(const char *filename, char *filelines[], int maxfilelines,
							    char fullbuf[], size_t fullbufsize);
int CALLING_CONVENTION getXsvKey(const char *fileLines[],
							     const char *suffix,
								 char keybuf[], size_t keybufsize);
int CALLING_CONVENTION strip(const char *instr, char retbuf[], size_t retbufsize);

int main(int argc, char **argv)
{
	FILE* fptr = 0;
	char 	*strLicenseKeyFileName = NULL;
	char 	*strLicenseKey = NULL;
	int		iDaysLeftThreshold = 12;
	long	lQuotaUsageToDate = 0;
	long	lQuotaThreshold = 12;
	int		iDaysGracePeriod = 0; 
	long	lQuotaGrace = 0;
	long	lCurrUsageValue = 0;
	int		iStatusCode = 0;
	char	*strUserHostName;
	void *ezLicInfoInstance = 0;
	char	*suffix = NULL;
	char filebuf[4096];
	char *filelines[200];
	int numlines = 0;
	char keybuf[2048];
	char keycookie1[1000];
	char keycookie2[1000];
	/* ezLicInfoInstance.iWarningsBitmap = 0; */


	if (argc < 3) {
		printf("\n*** usage: %s keyfile user-or-host-name ***", argv[0]);
		printf("\n    Example: %s keypub.csv user1\n", argv[0]);
		exit(2);
	}
	strLicenseKeyFileName = argv[1];
	strUserHostName = argv[2];
	suffix = getFileNameSuffix(strLicenseKeyFileName);

	numlines = 
		readFile(strLicenseKeyFileName, filelines, 200,
				filebuf, sizeof(filebuf));
	if (numlines < 0) {
		printf("*** Cannot process license key file due to errors.  Aborting. ***\n");
		exit(1);
	} else if (!numlines) {
		printf("License key file is empty.  Nothing to do.  Exiting.\n");
		exit(0);
	}
	strLicenseKey = &keybuf[0];
	if (getKey(filelines, numlines, filebuf, strlen(filebuf),
			   suffix, keybuf, sizeof(keybuf)) < 0)
		exit(1);

	printf("\nHello EasyLicenser World.\nChecking license key ......\n");

	/* printf("\nstrLicenseKey = %s\n", strLicenseKey); */
	ezLicInfoInstance = allocLicenseInfo();
	iStatusCode = checkLicenseKey(ezLicInfoInstance, 
								   keybuf, 
								   0, 0,
								   iDaysLeftThreshold, 
								   lQuotaUsageToDate,     
								   lQuotaThreshold, 
								   iDaysGracePeriod, 
								   lQuotaGrace, 	
								   strUserHostName, 
								   lCurrUsageValue);

	/* printf("\n returned from check lic");	*/
	if (iStatusCode < 0)
	{
		checkErrors(iStatusCode);
	}
	else
	{	
		checkWarnings(iStatusCode);
		printf("\nLicense key validation successful\n");
		displayLicenseParams(ezLicInfoInstance);
		printf("\nEnd Hello EasyLicenser World demo.\n");
	}
	freeLicenseInfo(ezLicInfoInstance);
	printf("\nTry secure license key check sequence.\n");
	printf("1: Seed a key cookie.\n");
	if (createKeyCookieSeed(strUserHostName, keycookie1, sizeof(keycookie1)) < 0) {
failret:
		printf("*** Failed.  Aborting. ***\n");
		exit(1);
	}
	printf("   Key cookie seed: %s\n", keycookie1);
	printf("2: Do secure key check with this cookie.  Should succeed.\n");
	if (checkKeySecure(keybuf, strUserHostName, 
					   keycookie1, keycookie2, 
					   sizeof(keycookie2)) < 0)
		goto failret;
	printf("3: Do secure key check with returned cookie.  Should succeed.\n");
	if (checkKeySecure(keybuf, strUserHostName, 
					   keycookie2, keycookie1, 
					   sizeof(keycookie1)) < 0)
		goto failret;
	printf("4: Do secure key check with bad cookie.  Should fail.\n");
	strcpy(keycookie1, "oiuyaioufois");
	if (checkKeySecure(keybuf, strUserHostName, 
					   keycookie1, keycookie2, 
					   sizeof(keycookie2)) < 0) {
		printf("*** Failed: key check succeeded when it should have failed. ***\n");
		goto failret;
	} else
		printf("Ok.  Failed as expected.\n");
	return 0;
}


int CALLING_CONVENTION checkKeySecure(
							const char *keybuf,
							const char	*userhost,
							char		keycookie[],
							char		newkeycookie[],
							size_t		keycookiebufsize)
{
	void *ezLicInfoInstance = allocLicenseInfo();
	int iStatusCode = checkLicenseKeySecure(
								   ezLicInfoInstance, 
								   keybuf, 
								   keycookie,
								   newkeycookie,
								   keycookiebufsize,
								   0, 0,
								   0, 
								   0,
								   0, 
								   1000, 
								   10000, 	
								   userhost, 
								   0);

	/* printf("\n returned from check lic");	*/
	if (iStatusCode < 0)
	{
		checkErrors(iStatusCode);
	}
	else
	{	
		checkWarnings(iStatusCode);
		printf("\nSecure license key validation successful\n");
	}
	freeLicenseInfo(ezLicInfoInstance);
	return 0;
}


/*
*	This function takes the Warnings Bit map as argument and interprets it and 
*	displays corresponding messages.
*	@param iWarningsBitMap : BitMap denoting Warnings.
*	@return statuscode
*/
int CALLING_CONVENTION checkWarnings(int iWarningsBitMap)
{
	/*
	if (DEBUG_PRINT)
	{
		printf("\n               iWarningsBitMap = %d ", iWarningsBitMap);
	}
	*/	

    if (iWarningsBitMap & EZLIC_WARN_EXPIRING)
    {
        printf("\nWarning: Time limited license key is about to expire.\n");
    }

	if (iWarningsBitMap & EZLIC_WARN_QUOTA)
    {
        printf("\nWarning: Usage limited license key is about to expire.\n");
    }	   
	if (iWarningsBitMap & EZLIC_WARN_EXPIRED)
    {
        printf("\nWarning: Time limited license has expired and is now in grace period.\n");
    }
	if (iWarningsBitMap & EZLIC_WARN_QUOTA_EXCEEDED)
    {
        printf("\nWarning: Usage limited license key has expired and is now in grace period.\n");
    }
    
	return iWarningsBitMap;
	
} /* End of the function check warnings */


/*
*	This function checks for variuos error codes returned from the library.
*/
int CALLING_CONVENTION checkErrors(int iWarningsBitMap)
{
	/* printf("\nEntered the check errors fucntion iWarningsBitMap = %d", iWarningsBitMap); */

	switch(iWarningsBitMap)
	{
			/**
			* Base exception code (invalid)
			*/
	case     EZLIC_EXC_BASE:

			printf("\nError: General license validation failure. Please check your license key.\n");
			break;

			/**
			* Malformed license key
			*/
	case     EZLIC_EXC_MALFORMED_KEY:

			printf("\nError: License key generation was incorrectly done.\n");
			break;

			/**
			* Expired license key
			*/
	case     EZLIC_EXC_EXPIRED:

			printf("\nError: The license has expired.\n");
			break;

			/**
			* Exceeded license quota
			*/
	case     EZLIC_EXC_EXCEEDED_LIMITS:

			printf("\nError: The License has exceeded the specified limits\n");
			break;

			/**
			* Invalid host or user or network name
			*/
	case     EZLIC_EXC_INVALID_HOST_USER_NET:

			printf("\nError: The machine name/user name is invalid.\n");
			break;

			/**
			* Invalid custom key
			*/
	case     EZLIC_EXC_INVALID_CUSTOM_KEY:

			printf("\nError: The custom key is invalid.\n");
			break;

			/**
			* Unimplemented feature
			*/
	case     EZLIC_EXC_UNIMPLEMENTED:

			printf("\nError: The feature is not implemented.\n");
			break;

			/**
			* Key generation error condition
			*/
	case     EZLIC_EXC_GEN:

			printf("\nError: Key generation execption (you shouldn't receive this error).\n");
			break;

			/**
			* Vendor key parse error condition
			*/
	case     EZLIC_EXC_VENDOR:

			printf("\nError: Vendor license key exception.\n");
			break;

			/**
			* Internal error condition
			*/
	case     EZLIC_EXC_INTERNAL:

			printf("\nError: internal exception.\n");
			break;

			/**
			* OS error
			*/
	case     EZLIC_EXC_OS:

			printf("\nError: OS exception.\n");
			break;

			/**
			* EasyLicenser configuration error
			*/
	case     EZLIC_EXC_CONFIG:

			printf("\nError: exception in configuration (you shouldn't get this error)\n");
			break;

			/**
			* Unlicensed option
			*/
	case     EZLIC_EXC_UNLICENSED_OPTION:

			printf("\nError: The option is unlicensed.\n");
			break;
	}
    
    if (iWarningsBitMap < 0)
    {
        printf("\n*** License key is invalid ***\n");
        return iWarningsBitMap;
    }
	printf("\nSuccess in verifying the license.\n");
	return 0;
}

void CALLING_CONVENTION showLicenseModel(int licenseModel,
										 int licenseType)
{
	printf("\n\tLicense Model:\t");
	switch(licenseModel) {
	case EZLIC_MODEL_USER:
		printf("User\n\tLicense Type:\t");
		switch(licenseType) {
		case EZLIC_TYPE_U_USER:
			printf("Named User\n");
			break;
		case EZLIC_TYPE_U_NODE:
			printf("Node Locked\n");
			break;
		case EZLIC_TYPE_U_FLOAT:
			printf("Floating User\n");
			break;
		default:
			printf("*** Unknown license type %d ***\n", licenseType);
		}
		break;
	case EZLIC_MODEL_SERVER:
		printf("Server\n\tLicense Type:\t");
		switch(licenseType) {
		case EZLIC_TYPE_SVR_CONC:
			printf("Concurrent User\n");
			break;
		case EZLIC_TYPE_SVR_CPU:
			printf("CPU Count\n");
			break;
		case EZLIC_TYPE_SVR_MHZ:
			printf("CPU MHZ\n");
			break;
		case EZLIC_TYPE_SVR_NMU:
			printf("Named User\n");
			break;
		default:
			printf("*** Unknown license type %d ***\n", licenseType);
		}
		break;
	default:
		printf("*** Unknown license model %d ***\n", licenseModel);
	}
}


/*
* This function displays the basic license parameters.
*/
void CALLING_CONVENTION displayLicenseParams(
							void* ezLicInstance)
{
	int value = 0;
	long lvalue = 0L;
	long ltime = 0;
	char* strValue = 0;


	printf("\nLicense Key Information\n");
	printf("\n=======================\n");


	strValue = getLicenseKey(ezLicInstance);
	if (strValue !=0)
	{
		printf("\n\tLicense Key:\t[%s]\n", strValue);
	}

	value = getLicenseModelCode(ezLicInstance);
	showLicenseModel(value, getLicenseTypeCode(ezLicInstance));
	if (getEnforce(ezLicInstance))
		printf("\tOperating System check enforced.\n");

	value = getLicenseModeBitmap(ezLicInstance);
	printf("\tLicense Mode Bitmap:\t0x%x\n", value);
	if (value & EZLIC_MODE_TIME)
		printf("\t\tTIME LIMITED\n");
	if (value & EZLIC_MODE_OPTIONS)
		printf("\t\tOPTIONS\n");
	if (value & EZLIC_MODE_METERED)
		printf("\t\tMETERED\n");
	if (value & EZLIC_MODE_CUSTOM_KEY)
		printf("\t\tCUSTOM KEY\n");
	if (value & EZLIC_MODE_CUSTOM_COOKIE)
		printf("\t\tCUSTOM COOKIE\n");

	lvalue = getUsageValue(ezLicInstance);
	if (lvalue > 0)
		printf("\tUsage Limit Value:\t%ld\n", lvalue);

	lvalue = getQuotaValue(ezLicInstance);
	if (lvalue > 0)
		printf("\tUsage Limited.  Quota Value: %ld\n", 
				lvalue);

	strValue = getOptions(ezLicInstance);
	if (strValue != 0 && strcmp(strValue, ""))
	{
		printf("\tOptions:\t[%s]\n", strValue);
	}

	ltime = getExpirationTime(ezLicInstance);

	if (ltime != -1)
	{
		struct tm when;
		when = *gmtime( &ltime );
		printf( "\tTime Limited.  Expiration Date: %s\n", 
				asctime(&when));
	}
   	
	strValue = getUserHostNetName(ezLicInstance);
	if (strValue !=0)
	{
		printf("\tUser/Host Name:\t[%s]\n", 
			strValue);
	}

	strValue = getCustomCookie(ezLicInstance);
	if (strValue !=0 && strValue[0] != '\0')
	{
		printf("\tCustom Cookie:\t[%s]\n", strValue);
	}

	strValue = getCustomKey(ezLicInstance);
	if (strValue !=0 && strValue[0] != '\0')
	{
		printf("\tCustom Key:[%s]\n", strValue);
	}
}


char * CALLING_CONVENTION getFileNameSuffix(const char *fname)
{
	char *suffp = strrchr(fname, (int)'.');
	if (suffp > fname && suffp > &fname[strlen(fname)-4])
		return NULL;
	return suffp+1;
}


int CALLING_CONVENTION getKey(const char *filelines[], int numlines,
							  const char fullbuf[], size_t fullbuflen, 
							  const char *suffix,
							  char keybuf[], size_t keybufsize)
{
	char *keyp = NULL;
	char *keyp2 = NULL;
	if (numlines <= 0)
		return -1;
	if (!strcmp(suffix,"xml")) {
		keyp = fullbuf;
		if (strncmp(keyp,"<?xml", strlen("<?xml")) ||
		    !(keyp = strstr(keyp, "<key id=")) ||
		    !(keyp = strchr(keyp, (int)'>'))) {
badxml:
			printf("\n*** Not an EasyLicenser XML file.  Aborting. ***\n");
			return -1;
		}
		keyp++;
		keyp2 = strstr(keyp, "</key>");
		if (strstr(keyp2, "<key id=")) {
			printf("\n*** File has multiple keys.  Only the first key will be checked. ***\n");
		}
		*keyp2 = '\0';
		return strip(keyp, keybuf, keybufsize);
	}
	if (!strcmp(suffix,"txt")) {
		if (!((keyp = strstr(fullbuf, "License Key List:")) &&
			  (keyp = strstr(fullbuf, "License key:")))) {
			printf("Didn't find License Key List / License key: \n");
badtxt:
			printf("\n*** Not an EasyLicenser-generated key text file.  Aborting. ***\n");
			return -1;
		}
		keyp = strchr(keyp, (int)':')+1;
		if (strstr(keyp, "License key:")) {
			printf("\n*** File has multiple keys.  Only the first key will be checked. ***\n");
		}
		if (keyp2 = strstr(keyp, "Key Id:"))
			*keyp2='\0';
		return strip(keyp, keybuf, keybufsize);
	}
	if (!strcmp(suffix,"ini")) {
		if (!(keyp = strchr(filelines[0], (int)'='))) {
			printf("\n*** Invalid property file format.  Aborting. ***\n");
			return -1;
		}
		return strip(keyp+1, keybuf, keybufsize);
	}
	if (!strcmp(suffix,"key")) {
		return strip(filelines[0], keybuf, keybufsize);
	}
	if (!(strcmp(suffix,"csv") && strcmp(suffix,"tab"))) {
		return getXsvKey(filelines, suffix,
						 keybuf, keybufsize);
	}
	return strip(fullbuf, keybuf, keybufsize);
}


int CALLING_CONVENTION readFile(const char *filename, char *filelines[], int maxfilelines,
							    char fullbuf[], size_t fullbufsize)
{
	int numlines = 0;
	int totlen = 0;
	int len = 0;
	char linebuf[1024];
	char *linep = NULL;
	char *lp = NULL;
	int trunc = 0;
	FILE *fh = fopen(filename,"r");
	if (!fh) {
		printf("\n*** Cannot open license key file '%s' ***\n",
				filename);
		return -1;
	}
	while (numlines <= maxfilelines &&
	       fgets(linebuf, sizeof(linebuf), fh)) {
		linep = (char *)malloc(sizeof(linebuf));
		lp = &linebuf[0];
		while (*lp && isspace((int)*lp))
			lp++;
		if (*lp) {
			strcpy(linep, lp);
			filelines[numlines++] = linep;
			len = strlen(linep);
			if (totlen + len < fullbufsize) {
				totlen += sprintf(&fullbuf[totlen], "%s ", linep);
			} else {
				printf("\n*** Warning: IO buffer overflow, truncating.\n");
				trunc = 1;
			}
		}
	}
	if ((!trunc) && numlines > maxfilelines)
		printf("\n*** Warning: IO buffer overflow, truncating.\n");
	fclose(fh);
	return numlines;
}


int CALLING_CONVENTION getXsvKey(const char *fileLines[],
							     const char *suffix,
								 char keybuf[], size_t keybufsize)
{
	char separator = !strcmp(suffix,"csv") ? ',' : '\t';
	char *firstLine = fileLines[0];
	char *keyptr = strrchr(firstLine, (int)separator);
	if (!keyptr) {
		printf("\n*** Invalid %s file.  Aborting. ***\n", suffix);
		return -1;
	}
	if (strlen(keyptr) >= keybufsize) {
		printf("\n*** Internal buffer overflow error.  Aborting. ***\n");
		return -1;
	}
	strcpy(keybuf, keyptr);
	return 0;
}


int CALLING_CONVENTION strip(const char *instr, char retbuf[], size_t retbufsize)
{
	char *outptr = &retbuf[0];
	int len = 0;
	char ch = 0;
	if (retbufsize <= strlen(instr))
		return -1;
	while (ch = *instr++) {
		if (!isspace((int)ch))
			*outptr++ = ch;
	}
	*outptr = '\0';
	return 0;
}

