
#ifndef __EZLICENSEINFO_H
#define  __EZLICENSEINFO_H

#include <ezlmplat.h>

#include <ezLicStatusCodes.h>
#include <ezLicLimits.h>
#include <ezLicCustomKeyHandler.h>

#ifdef __cplusplus
extern "C" {
#endif


/**
* License models ::
*/
/**
* 1. Single-user license model
*/
#define EZLIC_MODEL_USER 1
/**
* 2. Server license model
*/
#define EZLIC_MODEL_SERVER 2

/**
* Single-user license model: license types
*/
/**
* 1. User-locked
*/
#define EZLIC_TYPE_U_USER  1
/**
* 2. Node-locked
*/
#define EZLIC_TYPE_U_NODE  2
/**
* 3. Floating
*/
#define EZLIC_TYPE_U_FLOAT 3

/**
* Server license model : license types
*/
/**
* 1. Concurrent user
*/
#define EZLIC_TYPE_SVR_CONC  1
/**
* 2. CPU count
*/
#define EZLIC_TYPE_SVR_CPU  2
/**
* 3. CPU MHZ
*/
#define EZLIC_TYPE_SVR_MHZ 3
/**
* 4. Named-user
*/
#define EZLIC_TYPE_SVR_NMU 4

/**
* License modes - intended to be OR'ed into a bit map
*/
/**
* Time limited
*/
#define EZLIC_MODE_TIME 1
/**
* Metered
*/
#define EZLIC_MODE_METERED 2
/**
* Option-enabling
*/
#define EZLIC_MODE_OPTIONS 4
/**
* Custom key
*/
#define EZLIC_MODE_CUSTOM_KEY 8
/**
* Custom cookie
*/
#define EZLIC_MODE_CUSTOM_COOKIE 16



/* License state management */


/**
* Allocate license info
*/
DLLEXPORT void * CALLING_CONVENTION allocLicenseInfo();

/*
* Free license info
* @param licenseInfo: pointer to license info context to be freed
*/
DLLEXPORT void CALLING_CONVENTION freeLicenseInfo(void *licenseInfo);


/**
* returns license key from license info instance
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @return: pointer to encoded license key string
*/
DLLEXPORT char * CALLING_CONVENTION getLicenseKey(void *ezLicInfoInstance);

/**
* returns license model code
* (put comments here properly...)
*/
DLLEXPORT int CALLING_CONVENTION getLicenseModelCode(void *ezLicInfoInstance);

/**
* returns license type code
*/
DLLEXPORT int CALLING_CONVENTION getLicenseTypeCode(void *ezLicInfoInstance); 

/**
* returns license mode bitmap,
* bits set per EZLIC_MODE_*.
*/
DLLEXPORT int CALLING_CONVENTION getLicenseModeBitmap(void *ezLicInfoInstance); 

/**
* returns custom key contained in 
* the license key, null if none
*/
DLLEXPORT char * CALLING_CONVENTION getCustomKey(void *ezLicInfoInstance); 

/**
* returns custom cookie contained in 
* the license key, null if none
*/
DLLEXPORT char * CALLING_CONVENTION getCustomCookie(void *ezLicInfoInstance); 

/**
* returns user / host name, null if none
*/
DLLEXPORT char * CALLING_CONVENTION getUserHostNetName(void *ezLicInfoInstance); 

/**
* returns expiration date, null if none
*/
DLLEXPORT long CALLING_CONVENTION getExpirationTime(void *ezLicInfoInstance);

/**
* returns strOptions string, null if none
*/
DLLEXPORT char * CALLING_CONVENTION getOptions(void *ezLicInfoInstance); 

/**
* returns quota limit value, -1 if no quota limit.
*/
DLLEXPORT long CALLING_CONVENTION getQuotaValue(void *ezLicInfoInstance); 

/**
* returns usage limit value (cpu / CPU MHZ etc.) for
* the license key, -1 if no limit.
*/
DLLEXPORT long CALLING_CONVENTION getUsageValue(void *ezLicInfoInstance); 

/**
* returns warning bitmap for license key
*/
DLLEXPORT int CALLING_CONVENTION getWarningsBitmap(void *ezLicInfoInstance); 

/**
* sets license key
*/
DLLEXPORT void CALLING_CONVENTION setLicenseKey(void *ezLicInfoInstance, const char* strLicensekey); 

/**
* sets license model code
*/
DLLEXPORT void CALLING_CONVENTION setLicenseModelCode(void *ezLicInfoInstance, const int  iNewLicenseModelCode); 

/**
* sets license type code
*/
DLLEXPORT void CALLING_CONVENTION setLicenseTypeCode(void *ezLicInfoInstance, const int  iNewLicenseTypeCode); 

/**
* sets license mode bitmap
*/
DLLEXPORT void CALLING_CONVENTION setLicenseModeBitmap(void *ezLicInfoInstance, const int  iNewLicenseModeBitmap);

/**
* sets custom key value
*/
DLLEXPORT void CALLING_CONVENTION setCustomKey(void *ezLicInfoInstance, const char* strNewCustomKey); 

/**
* sets custom cookie value
*/
DLLEXPORT void CALLING_CONVENTION setCustomCookie(void *ezLicInfoInstance, const char* strNewCustomCookie); 

/**
* sets user/host name according to context
*/
DLLEXPORT void CALLING_CONVENTION setUserHostNetName(void *ezLicInfoInstance, const char* strNewUserHostNetName); 

/**
* sets license expiration date
*/
DLLEXPORT void CALLING_CONVENTION setExpirationTime(void *ezLicInfoInstance, const long lNewExpirationTime); 

/**
* sets strOptions string
*/
DLLEXPORT void CALLING_CONVENTION setOptions(void *ezLicInfoInstance, const char* strNewOptions); 

/**
* sets usage quota value
*/
DLLEXPORT void CALLING_CONVENTION setQuotaValue(void *ezLicInfoInstance, const long lNewQuotaValue); 

/**
* sets value for usage limit
*/
DLLEXPORT void CALLING_CONVENTION setUsageValue(void *ezLicInfoInstance, const long lNewUsageValue); 

/**
* sets warning bitmap
*/
DLLEXPORT void CALLING_CONVENTION setWarningsBitmap(void *ezLicInfoInstance, const int  iNewWarningsBitmap);

/**
* createKeyCookieSeed:
* Create a key cookie seed that is the first key cookie
* passed into the "checkLicenseKeySecure" API call
* by an application.  An application invokes this
* method the first time it is executed after receiving
* a license key, and saves the returned key cookie in
* persistent store.  Subsequently, the value in the persistent
* store is supplied to the "checkLicenseKeySecure" API call,
* which upon a successful license key check returns an updated
* key cookie that the application replaces the previous value
* with.  The combination of the "createKeyCookieSeed" and
* "checkLicenseKey" API's enables an application to be both
* spoof-proof and protect itself from hackers turning back
* the system clock to re-run time-limited copies of programs.
* @param userHost: the user or host name corresponding to the key
* @param strKeyCookieSeedBuf: string buffer to receive generated 
*                  key cookie string.
* @param keyCookieSeedBufsize: size of key cookie seed buffer.
* @return 0 if success, -1 if error (buffer overflow)
*/
DLLEXPORT int CALLING_CONVENTION createKeyCookieSeed(const char *userHost,
									char strKeyCookieSeedBuf[],
									size_t keyCookieSeedBufsize);

/* API calls for VB interfacing */


/**
* getLicenseKeyStr:
* returns license key from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return license key
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getLicenseKeyStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);



/**
* getCustomKeyStr:
* returns custom license key from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return custom key
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getCustomKeyStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);



/**
* getCustomCookieStr:
* returns custom cookie from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return custom cookie
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getCustomCookieStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* getUserHostNetNameStr:
* returns user/host/network from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return user / host / network name
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getUserHostNetNameStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);

/**
* getExpirationTimeStr:
* returns expiration date/time string from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return expiration date string
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getExpirationTimeStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* getOptionsStr:
* returns options string from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return options string
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getOptionsStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* getEnforce:
* returns indicator for whether user / host check is enforced.
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @return: 0 if not enforced, 1 if enforced
*/
DLLEXPORT int CALLING_CONVENTION getEnforce(void *ezLicInfoInstance);



/* License Key Check API calls */

/**
* Check the specified license key against the provided info for key violations.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*   @return warning code bitmap to indicate almost-out-of-quota, expiring, etc., or exception code if negative.
*/
DLLEXPORT int CALLING_CONVENTION checkSingleUserLicenseKeyBasic (void* ezLicInfoInstancep, 
													    const char* 	strEncodedLicenseKey,
														const int		iDaysLeftThreshold,
														const long		lQuotaUsageToDate,
														const long		lQuotaThreshold,
														const int 		iDaysGracePeriod,
														const long		lQuotaGrace,
														const char*		strUserHostName);

/**
* Check the specified license key against the provided info for key violations.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encrypted form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @return warning code bitmap to indicate almost-out-of-quota, expiring, etc.
*/
DLLEXPORT int CALLING_CONVENTION checkMultiUserLicenseKeyBasic (void*				ezLicInfoInstancep, 
				        							  const char* 		strEncodedLicenseKey,
													   const int 		iDaysLeftThreshold,
													   const long		lQuotaUsageToDate,
													   const long		lQuotaThreshold,
													   const int 		iDaysGracePeriod,
													   const long		lQuotaGrace, 
													   const char*		strUserHostName, 
													   const long		lCurrUsageValue);

/**
* (Full version:) 
* Check the specified license key against the provided info for key violations.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @return warning code bitmap to indicate almost-out-of-quota, expiring, etc.
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKey(void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue);


/**
* (Secure version:) 
* Check the specified license key against the provided info for key violations,
* and set the context based on the key.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*   @param strKeyCookie: the key cookie that was returned
*          		from the previous call to this method for 
*          		this key, or from the createKeyCookieSeed() 
*          		method if none.
*   @param strNewKeyCookieBuf:			Pointer to buffer to receive new key cookie value.
*   @param newKeyCookieBufsize:     Size of new key cookie buffer
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @return warning code bitmap to indicate almost-out-of-quota, expiring, etc.
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKeySecure(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   const char*		strKeyCookie,
									   char				strNewKeyCookieBuf[],
									   size_t			strNewKeyCookieBufsize,
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue);


#ifdef __cplusplus
}
#endif

#endif

	
