/*
 * @(#)jdemo.java    1.00   02/21/02
 *
 * Copyright (c) 2002, 2003 Agilis Software LLC, Fremont, CA, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Agilis Software LLC ("Confidential Information").  You shall not
 * disclose such Confidential Information and shal use it only
 * in accordance with the terms of the license agreement you entered
 * into with Agilis Software LLC.
 */

import java.io.*;
import java.util.*;
import com.vs.ezlicrun.*;

public class jdemo {

	static final int KEY_MAXLENGTH = 800;

	public jdemo() {}

	/**
	 * main entry point:
	 * Accepts two command line arguments:
	 * 1. Name of a text file containing a lincense key to be validated.
	 * 2. Expected user / host name that should match what's in the key.
	 * Checks the license key and displays its license parameters if it
	 * passes the check.
	 */
	public static void main(String[] args) {
		if (args.length < 2) {
			System.out.println("*** Usage: JDemo key-text-file-name user-host name ***");
			System.exit(1);
		}
		String keyfile = args[0];
		String userhost = args[1];
		File kf = new File(keyfile);
		if (!kf.exists()) {
			System.out.println("*** License key file '" +
						keyfile + "' not found ***");
			System.exit(2);
		}
		String keytext = "";
		BufferedReader in = null;
		try {
			in = new BufferedReader(
					new InputStreamReader(
						new FileInputStream(kf)));
			String keyline = null;
			while ((keyline = in.readLine()) != null)
				keytext += "\n"+keyline;
	 		if ((keytext = getKey(keytext, getFilenameSuffix(keyfile))) == null)
				System.exit(1);
			if (keytext.length() > KEY_MAXLENGTH) {
				System.out.println("*** Invalid key: string too long. ***");
				System.exit(1);
			}
		} catch (Exception ex) {
			System.out.println("*** IO Error reading license key file ***");
			System.out.println("Exception: " + ex.getClass().getName() + ", Message: " + ex.getMessage());
			ex.printStackTrace();
			System.exit(2);
		} finally {
			try {
				in.close();
			} catch (Exception exx) {
			}
		}
		System.out.println("\nHello EasyLicenser World.\n");
		System.out.println("Checking license key:\n"+keytext+"\n...\n\n");
		EzLicenseInfo licenseInfo = new EzLicenseInfo();
		int warnings = 0;
		try {
			warnings =
			licenseInfo.checkLicenseKey(
						keytext,
						null, null,
						1000, 0, 10000,
						100, 1000,
						userhost,
						0);
		} catch (Exception ex) {
			System.out.println("*** License key check failed.  Error:\n\t" +
				ex.getMessage());
			System.exit(3);
		}
		checkWarnings(warnings);
		displayLicenseParams(licenseInfo);
		System.out.println("\nLicense key validation successful.\n");
	}

	static void checkWarnings(int warnings)
	{
		if (warnings != 0) {
			System.out.println("\tWarnings:");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_EXPIRING) != 0)
				System.out.println("\t\tTime Limited Key Is About To Expire");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_QUOTA) != 0)
				System.out.println("\t\tQuota Limited Key Is About To Expire");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_EXPIRED) != 0)
				System.out.println("\t\tTime Limited Key Has Expired");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_QUOTA_EXCEEDED) != 0)
				System.out.println("\t\tQuota Limited Key Has Expired");
		}
	}

	static void displayLicenseParams(EzLicenseInfo licenseInfo)
	{
		System.out.println("\n\tLicense Key Information:\n\t========================\n\n");
		System.out.println("\t\tLicense key: [" + licenseInfo.getLicenseKey() + "]");
		showLicenseModel(licenseInfo);
		int bitmap = licenseInfo.getLicenseModeBitmap();
		System.out.println("\t\tLicense Mode Bitmap: " +
						bitmap);
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_TIME) != 0)
			System.out.println("\t\t\tTIME LIMITED");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_OPTIONS) != 0)
			System.out.println("\t\t\tOPTIONS");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_METERED) != 0)
			System.out.println("\t\t\tMETERED");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_CUSTOM_KEY) != 0)
			System.out.println("\t\t\tCUSTOM KEY");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_CUSTOM_COOKIE) != 0)
			System.out.println("\t\t\tCUSTOM COOKIE");
		long lvalue;
		lvalue = licenseInfo.getUsageValue();
		if (lvalue > 0)
			System.out.println("\t\tUsage Limit Value: " + lvalue);
		lvalue = licenseInfo.getQuotaValue();
		if (lvalue > 0)
			System.out.println("\t\tQuota Value: " + lvalue);
		String options = licenseInfo.getOptions();
		if (options != null && !options.equals(""))
			System.out.println("\t\tOptions: [" + options + "]");
		Date expDate = licenseInfo.getExpirationDate();
		if (expDate != null)
			System.out.println("\t\tExpiration Date: " +
							expDate.toString());
		System.out.println("\t\tUser / host name: " +
						licenseInfo.getUserHostNetName());
		if (licenseInfo.getEnforce())
			System.out.println("\t\tUser / host name operating system check enforced.");
		String customCookie = licenseInfo.getCustomCookie();
		if (customCookie != null && !customCookie.equals(""))
			System.out.println("\t\tCustom Cookie: [" + customCookie + "]");
		String customKey = licenseInfo.getCustomKey();
		if (customKey != null && !customKey.equals(""))
			System.out.println("\t\tCustom Key: [" + customKey + "]");
	}

	static void showLicenseModel(EzLicenseInfo licenseInfo)
	{
		int licenseModel = licenseInfo.getLicenseModelCode();
		int licenseType = licenseInfo.getLicenseTypeCode();
		System.out.print("\t\tLicense Model:\t");
		switch(licenseModel) {
		case EzLicenseInfo.EZLIC_MODEL_USER:
			System.out.print("User\n\t\tLicense Type:\t");
			switch(licenseType) {
			case EzLicenseInfo.EZLIC_TYPE_U_USER:
				System.out.println("Named User\n");
				break;
			case EzLicenseInfo.EZLIC_TYPE_U_NODE:
				System.out.println("Node Locked\n");
				break;
			case EzLicenseInfo.EZLIC_TYPE_U_FLOAT:
				System.out.println("Floating User\n");
				break;
			default:
				System.out.println("*** Unknown license type " + licenseType + " ***");
			}
			break;
		case EzLicenseInfo.EZLIC_MODEL_SERVER:
			System.out.print("Server\n\t\tLicense Type:\t");
			switch(licenseType) {
			case EzLicenseInfo.EZLIC_TYPE_SVR_CONC:
				System.out.println("Concurrent User\n");
				break;
			case EzLicenseInfo.EZLIC_TYPE_SVR_CPU:
				System.out.println("CPU Count\n");
				break;
			case EzLicenseInfo.EZLIC_TYPE_SVR_MHZ:
				System.out.println("CPU MHZ\n");
				break;
			case EzLicenseInfo.EZLIC_TYPE_SVR_NMU:
				System.out.println("Named User\n");
				break;
			default:
				System.out.println("*** Unknown license type " + licenseType + " ***");
			}
			break;
		default:
			System.out.println("*** Unknown license model " + licenseModel + " ***");
		}
	}

	static String getFilenameSuffix(String filename) {
		int dotpos = filename.lastIndexOf('.');
		if (dotpos <= 0 || dotpos > filename.length()-4) {
			return "";
		}
		return filename.substring(dotpos+1);
	}

	static String getKey(String fileContent, String suffix) {
		int keyidx = -1;
		fileContent = fileContent.trim();
		if (suffix.equalsIgnoreCase("xml")) {
			if (!(fileContent.startsWith("<?xml") &&
			      (keyidx = fileContent.indexOf("<key id=")) > 0)) {
				System.out.println("*** Not an EasyLicenser XML file.  Aborting. ***");
				return null;
			}
			fileContent = fileContent.substring(keyidx);
			fileContent = fileContent.substring(fileContent.indexOf('>')+1);
			keyidx = fileContent.indexOf("</key>");
			if (keyidx != fileContent.lastIndexOf("</key>")) {
				System.out.println("*** File has multiple keys.  Only first key will be checked. ***");
			}
			fileContent = fileContent.substring(0, keyidx);
			return strip(fileContent);
		}
		if (suffix.equalsIgnoreCase("txt")) {
			if (fileContent.startsWith("License Key List:")) {
				if ((keyidx = fileContent.indexOf("License key:")) < 0) {
					System.out.println("*** Not an EasyLicenser-generated text file.  Aborting. ***");
					return null;
				}
				fileContent = fileContent.substring(keyidx);
				fileContent = fileContent.substring(fileContent.indexOf(":")+1);
				if (fileContent.indexOf("License key:") > 0){
					System.out.println("*** File has multiple keys.  Only first key will be checked. ***");
				}
				keyidx = fileContent.indexOf("Key Id:");
				if (keyidx > 0)
					fileContent = fileContent.substring(0, keyidx);
			}
			return strip(fileContent);
		}
		if (suffix.equalsIgnoreCase("ini")) {
			if ((fileContent = getFirstLine(fileContent)) == null)
				return null;
			if ((keyidx = fileContent.indexOf((int)'=')) <= 0) {
				System.out.println("*** Invalid property file.  Aborting. ***");
				return null;
			}
			return strip(fileContent.substring(keyidx+1));
		}
		if (suffix.equalsIgnoreCase("csv") || suffix.equalsIgnoreCase("tab")) {
			return getXsvKey(fileContent, suffix);
		}
		if (suffix.equalsIgnoreCase("key")) {
			if ((fileContent = getFirstLine(fileContent)) == null)
				return null;
			return strip(fileContent);
		}
		// else assume vanilla pure-key file -- too bad if it isn't
		return strip(fileContent);
	}

	static String getXsvKey(String fileContent, String suffix) {
		char separator = suffix.equalsIgnoreCase("csv") ? ',':'\t';
		int keyidx = -1;
		if ((fileContent = getFirstLine(fileContent)) == null)
			return null;
		if ((keyidx = fileContent.lastIndexOf((int)separator)) <= 0) {
			System.out.println("*** Invalid " + suffix +
							" file.  Aborting. ***");
			return null;
		}
		return strip(fileContent.substring(keyidx+1));
	}

	static String getFirstLine(String fileContent) {
		int lf = fileContent.indexOf("\n");
		if (lf > 0) {
			System.out.println("*** File has multiple keys.  Only first key will be checked. ***");
			return fileContent.substring(0, lf);
		} else if (lf == 0) {
			System.out.println("*** File is empty.  Nothing to do. ***");
			return null;
		}
		return fileContent;
	}

	static String strip(String instr) {
		String retstr = "";
		try {
			StringTokenizer stk = new StringTokenizer(instr," \n\t");
			while (stk.hasMoreTokens()) {
				String tokstr = stk.nextToken();
				retstr += tokstr;
			}
		} catch (Exception ex) {
			System.out.println("*** Got exception: " + ex.getMessage());
		}
		return retstr;
	}
	
}
